﻿using System;
using System.Collections.Generic;
using System.Linq;
using MongoDB.Bson;
using MongoDB.Driver;
using MongoDB.Driver.Builders;
using MongoDB.Driver.Linq;

namespace mongoBooks
{
    // http://docs.mongodb.org/ecosystem/tutorial/getting-started-with-csharp-driver/
    class Program
    {
        private const string ConnectionString = "mongodb://localhost";
        private const string DatabaseName = "test";
        private static readonly Book SomeBook = new Book { title = "Yet another book", pages = 700 };


        static void Main()
        {
            MongoClient client = new MongoClient(ConnectionString);
            MongoServer server = client.GetServer();
            MongoDatabase database = server.GetDatabase(DatabaseName);
            MongoCollection<Book> collection = database.GetCollection<Book>("books");

            DoFind(collection);
            //DoFindUsingLinq(collection);
            //DoInsert(collection);
            //DoSave(collection);
            //DoUpdate(collection);
            //DoRemove(collection);
        }

       

        private static void DoFind(MongoCollection<Book> collection)
        {
            IMongoQuery query = Query<Book>.GT(book => book.pages, 530); // GT means "greater than"
            MongoCursor<Book> books = collection.Find(query);
            foreach (Book book in books)
            {
                Console.WriteLine(book);
            }
        }


        private static void DoFindUsingLinq(MongoCollection<Book> collection)
        {
            IQueryable<Book> books = from e in collection.AsQueryable() 
                                     where e.pages > 530 select e;
            //Console.WriteLine(books.Count());
            foreach (Book book in books)
            {
                Console.WriteLine(book);
            }
        }

        // http://docs.mongodb.org/ecosystem/tutorial/getting-started-with-csharp-driver/
        private static void DoInsert(MongoCollection<Book> collection)
        {
            collection.Insert(SomeBook);
            Console.WriteLine("Book inserted: _id: " + SomeBook.Id);
        }

        private static void DoSave(MongoCollection<Book> collection)
        {
            SomeBook.pages = 600;
            collection.Save(SomeBook);
            // Writes the full document back to the database
        }

        private static void DoUpdate(MongoCollection<Book> collection)
        {
            IMongoQuery query = Query<Book>.EQ(book => book.title, "Yet another book");
            var update = Update<Book>.Set(book => book.title, "Yet another book, 2nd edition");
            collection.Update(query, update);
        }

        private static void DoRemove(MongoCollection<Book> collection)
        {
            IMongoQuery query = Query<Book>.EQ(book => book.title, "Yet another book");
            WriteConcernResult writeConcernResult = collection.Remove(query);
            Console.WriteLine("Documents deleted: " + writeConcernResult.DocumentsAffected + " " + writeConcernResult.Response);
        }
    }

    class Book
    {
        public ObjectId Id { get; set; }

        // names of properties must be spelled as in the documents. Case matters!
        public String title { get; set; }

        public int pages { get; set; }

        public int publicationYear { get; set; }

        public String publisher { get; set; }

        public IEnumerable<Author> author { get; set; }

        public override string ToString()
        {
            return title + " " + pages;
        }
    }

    class Author
    {
        public String firstname { get; set; }
        public String lastname { get; set; }
    }
}
